﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ServiceProxies;

namespace ProxyLayer
{
    /// <summary>
    /// This class provides some adapter methods which allow our .Net code to call the BPM services simply without getting too tied to the 
    /// Implementation details of the services themselves.
    /// </summary>
    public class Adapter
    {

        private ServiceClientFactory _handler;

        public Adapter(string user, string password, string host, string port, bool secure)
        {
            _handler = new ServiceClientFactory(user, password, host, port, secure);
        }

        /// <summary>
        /// This method obtains the worklist for the user specified in the credentials which are passed in.
        /// </summary>
        /// <param name="credentials"></param>
        /// <returns></returns>
        public WorkItem[] GetWorkList(string guid)
        {
            getWorkListItems getItems = new getWorkListItems();

            getItems.entityID = new XmlModelEntityId();

            getItems.getTotalCount = true;
            getItems.numberOfItems = 100;
            getItems.startPosition = 0;

            getItems.entityID.entitytype = OrganisationalEntityType.RESOURCE;
            getItems.entityID.guid = guid;
            getItems.entityID.modelversion = -1;

            WorkListServiceClient wlSvcClient = _handler.GetWorkListServiceClient();

            getWorkListItemsResponse response = wlSvcClient.getWorkListItems(getItems);

            if (response != null)
            {
                if (response.workItems != null)
                {
                    WorkItem[] items = response.workItems;
                    return items;
                }

            }
            throw new Exception("Failed to get WorkList - call returned null");

        }

        /// <summary>
        /// This method acquires the GUID for the named resource.  In a lot of scenarios it acts a lot like a
        /// login too - because BPM services require a username/pw to make the call, the user credentials will be verified by doing this also.
        /// </summary>
        /// 
        public string GetGUID(string resource) 
        {

            EntityResolverServiceClient entityClient = _handler.GetEntityResolverClient();

            lookupUser user = new lookupUser();
            user.name = resource;
            user.getdetail = true;
            lookupUserResponse response = entityClient.lookupUser(user);
            return response.detail[0].guid;
        }


        /// <summary>
        /// This method opens the work item
        /// </summary>
        public workResponse OpenItem(string guid, WorkItem workItem)
        {

            WorkPresentationServiceClient wpSvcClient = _handler.GetWorkPresentationServiceClient();

            BaseWorkRequest request = new BaseWorkRequest();

            request.workItem = new WorkItem1();
            request.workItem.id = workItem.id.id;
            request.workItem.version = workItem.id.version;

            request.resourceId = guid;

            request.channelId = "openspaceGWTPull_DefaultChannel";
            request.channelType = ChannelType.openspaceChannel;
            request.responsePayloadMode = payloadModeType.JSON;
              
            workResponse response = wpSvcClient.openWorkItem(request);

            return response;

        }

        /// <summary>
        /// This method submits the work item, causing the process to be progressed
        /// </summary>
        public void SubmitWorkItem(WorkItem item, string workTypeUid, string workTypeVersion, string guid, string data)
        {

            WorkPresentationServiceClient wpSvcClient = _handler.GetWorkPresentationServiceClient();

            WorkRequest workRequest = new WorkRequest();
            workRequest.responsePayloadMode = payloadModeType.JSON;

            workRequest.resourceId = guid;

            workRequest.payloadDetails = new dataPayload();
            workRequest.payloadDetails.payloadMode = payloadModeType.JSON;
            workRequest.payloadDetails.Item = data;

            workRequest.channelId = "openspaceGWTPull_DefaultChannel";
            workRequest.channelType = ChannelType.openspaceChannel;

            workRequest.workItem = new WorkItem1();
            workRequest.workItem.id = item.id.id;
            workRequest.workItem.version = -1;

            workRequest.workTypeDetail = new BaseWorkRequestWorkTypeDetail();
            workRequest.workTypeDetail.uid = workTypeUid;
            workRequest.workTypeDetail.version = workTypeVersion;
            workRequest.workTypeDetail.openNextPiled = false;

            workResponse response = wpSvcClient.completeWorkItem(workRequest);
            
        }

        /// <summary>
        /// This method closes the work item
        /// </summary>
        public void CloseWorkItem(WorkItem item, string workTypeUid, string workTypeVersion, string guid, string data)
        {

            WorkPresentationServiceClient wpSvcClient = _handler.GetWorkPresentationServiceClient();

            WorkRequest workRequest = new WorkRequest();

            workRequest.resourceId = guid;

            workRequest.responsePayloadMode = payloadModeType.JSON;

            workRequest.payloadDetails = new dataPayload();
            workRequest.payloadDetails.payloadMode = payloadModeType.JSON;
            workRequest.payloadDetails.Item = data;

            workRequest.channelId = "openspaceGWTPull_DefaultChannel";
            workRequest.channelType = ChannelType.openspaceChannel;

            workRequest.workItem = new WorkItem1();
            workRequest.workItem.id = item.id.id;
            workRequest.workItem.version = -1;

            workRequest.workTypeDetail = new BaseWorkRequestWorkTypeDetail();
            workRequest.workTypeDetail.uid = workTypeUid;
            workRequest.workTypeDetail.version = workTypeVersion;
            workRequest.workTypeDetail.openNextPiled = false;
            
            workResponse response = wpSvcClient.closeWorkItem(workRequest);

        }

        /// <summary>
        /// This method closes the work item
        /// </summary>
        public void CancelWorkItem(WorkItem item, string workTypeUid, string workTypeVersion, string guid)
        {

            WorkPresentationServiceClient wpSvcClient = _handler.GetWorkPresentationServiceClient();

            WorkRequest workRequest = new WorkRequest();

            workRequest.resourceId = guid;

            workRequest.channelId = "openspaceGWTPull_DefaultChannel";
            workRequest.channelType = ChannelType.openspaceChannel;

            workRequest.workItem = new WorkItem1();
            workRequest.workItem.id = item.id.id;
            workRequest.workItem.version = -1;

            workRequest.workTypeDetail = new BaseWorkRequestWorkTypeDetail();
            workRequest.workTypeDetail.uid = workTypeUid;
            workRequest.workTypeDetail.version = workTypeVersion;
            workRequest.workTypeDetail.openNextPiled = false;

            workResponse response = wpSvcClient.cancelWorkItem(workRequest);

        }

        public string GetFormURL(workResponse response)
        {

            return _handler.GetBaseURL() + "bpmresources/" + response.presentation.formIdenitifier;
        }

        public string GetJSURL()
        {
            return _handler.GetBaseURL() + "bpmresources/formsclient/formsclient.nocache.js";
        }

        public string GetBOMJSPath(workResponse item)
        {
            string formURL = item.presentation.formIdenitifier;

            return _handler.GetBaseURL() + "bpmresources/" + formURL.Substring(0, formURL.IndexOf('/'));
        }
    }

}
