﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Net;
using System.Net.Security;

namespace ServiceProxies
{
    /// <summary>
    /// This class is the central entry point to acquire the BPM service clients as defined in the Services.cs
    /// It is used to intercept the calls and ensure that the endpoint/binding and behavior are set correctly
    /// </summary> 
    public class ServiceClientFactory
    {
        private string _host;
        private string _port;
        private bool _secure;

        private WorkPresentationServiceClient _wpsClient = null;
        private EntityResolverServiceClient _ersClient = null;
        private WorkListServiceClient _wlsClient = null;

        /// <summary>
        /// Default constructor which sets no security behavior up
        /// This will need to be set later by calling SecurityBehavior
        ///
        /// </summary>
        /// <param name="host"></param>
        /// <param name="port"></param>
        /// <param name="secure"></param>
        public ServiceClientFactory(string host, string port, bool secure)
        {
            // WARNING: The following code should not be used in a production environment as it causes the application to do no validation of
            // certificates issued by SSL servers. 
            ServicePointManager.ServerCertificateValidationCallback = new RemoteCertificateValidationCallback(
            delegate
            {
                return true;
            });
            _secure = secure;
            _host = host;
            _port = port;
        }

        /// <summary>
        /// Constructor that applies a default usernameBehavior model for security
        /// </summary>
        /// <param name="user"></param>
        /// <param name="password"></param>
        /// <param name="host"></param>
        /// <param name="port"></param>
        /// <param name="secure"></param>
        public ServiceClientFactory(string user, string password, string host, string port, bool secure) : this(host,port,secure)
        {
            SecurityBehavior = new UsernameBehavior(user, password);

        }

        public IEndpointBehavior SecurityBehavior
        {
            get;
            set;
        }
        /// <summary>
        /// Getter for the EntityResolverClient
        /// </summary>
        /// <returns></returns>
        public EntityResolverServiceClient GetEntityResolverClient()
        {
            if (_ersClient == null)
            {

                _ersClient = new EntityResolverServiceClient(CreateHttpBinding("EntityResolverService.soap"), new EndpointAddress(GetEntityUrl()));

                _ersClient.Endpoint.Behaviors.Add(SecurityBehavior);
            }
            return (_ersClient);

        }

        public WorkPresentationServiceClient GetWorkPresentationServiceClient()
        {
            if (_wpsClient == null)
            {
                _wpsClient = new WorkPresentationServiceClient(CreateHttpBinding("WorkPresentationService.soap"), new EndpointAddress(GetWorkPresentationServiceUrl()));
                _wpsClient.Endpoint.Behaviors.Add(SecurityBehavior);
            }

            return _wpsClient;
        }

        /// <summary>
        /// Getter for the WorkListServiceClient
        /// </summary>
        /// <returns></returns>
        public WorkListServiceClient GetWorkListServiceClient()
        {
            if (_wlsClient == null)
            {
                _wlsClient = new WorkListServiceClient(
                                        CreateHttpBinding("WorkListService.soap"),
                                        new EndpointAddress(GetWorkListServiceUrl()));

                _wlsClient.Endpoint.Behaviors.Add(SecurityBehavior);
            }
            return (_wlsClient);
        }



        /// <summary>
        /// This method returns the URL for the WorklistService
        /// </summary>
        /// <returns></returns>
        private string GetWorkPresentationServiceUrl()
        {
            return GetUrl("amxbpm/WorkPresentationService");
        }


        /// <summary>
        /// This method returns the URL for the WorklistService
        /// </summary>
        /// <returns></returns>
        private string GetWorkListServiceUrl()
        {
            return GetUrl("amxbpm/WorkListService");
        }

        /// <summary>
        /// This method returns the URL for the EntityResolverService
        /// </summary>
        /// <returns></returns>
        private string GetEntityUrl()
        {
            return GetUrl("amxbpm/EntityResolverService");
        }

        /// <summary>
        /// This method creates an HTTP Binding for the named host/security
        /// This allows us to dynamically select these at runtime
        /// </summary>
        /// <returns></returns>
        private BasicHttpBinding CreateHttpBinding(String name)
        {

            BasicHttpBinding binding;
            if (_secure)
            {
                binding = new BasicHttpBinding(BasicHttpSecurityMode.Transport);
                binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.None;

            }
            else
            {
                binding = new BasicHttpBinding(BasicHttpSecurityMode.None);
            }

            binding.Name = name;
            return binding;
        }


        /// <summary>
        /// Generic method for generating a URL with the named host settings
        /// </summary>
        /// <param name="endpoint">Endpoint for the URL</param>
        /// <param name="host">Hostname for the URL</param>
        /// <param name="secure">Whether or not the URL is secure</param>
        /// <returns></returns>
        private string GetUrl(string endpoint)
        {
            string protocol;
            if (_secure)
            {
                protocol = "https://";
            }
            else
            {
                protocol = "http://";
            }

            return protocol + _host + ":"+_port+"/" + endpoint;
        
        }





        internal string GetBaseURL()
        {
            return GetUrl("");
        }
    }
}
